@@ -1,6 +1,7 @@
 import asyncHandler from 'express-async-handler';
 import { Op, QueryTypes, literal } from 'sequelize';
-import { NotFoundError, InvalidParameterError, InvalidOperationError } from '@tamanu/shared/errors';
+import { subject } from '@casl/ability';
+import { NotFoundError, InvalidParameterError, InvalidOperationError } from '@tamanu/errors';
 import { getCurrentDateTimeString } from '@tamanu/utils/dateTime';
 import config from 'config';
 import { toCountryDateTimeString } from '@tamanu/shared/utils/countryDateTime';
@@ -14,7 +15,7 @@ import {
   IMAGING_REQUEST_STATUS_TYPES,
   TASK_STATUSES,
   SURVEY_TYPES,
-  DASHBOARD_ONLY_TASK_TYPES
+  DASHBOARD_ONLY_TASK_TYPES,
 } from '@tamanu/constants';
 import {
   simpleGet,
@@ -27,7 +28,9 @@ import {
 } from '@tamanu/shared/utils/crudHelpers';
 import { add } from 'date-fns';
 import { z } from 'zod';
+import { keyBy } from 'lodash';
 
+import { createEncounterSchema } from '@tamanu/shared/schemas/facility/requests/createEncounter.schema';
 import { uploadAttachment } from '../../utils/uploadAttachment';
 import { noteChangelogsHandler, noteListHandler } from '../../routeHandlers';
 import { createPatientLetter } from '../../routeHandlers/createPatientLetter';
@@ -39,6 +42,7 @@ import {
   deleteSurveyResponse,
 } from '../../routeHandlers/deleteModel';
 import { getPermittedSurveyIds } from '../../utils/getPermittedSurveyIds';
+import { validate } from '../../utils/validate';
 
 export const encounter = softDeletionCheckingRouter('Encounter');
 
@@ -48,7 +52,8 @@ encounter.post(
   asyncHandler(async (req, res) => {
     const { models, body, user } = req;
     req.checkPermission('create', 'Encounter');
-    const encounterObject = await models.Encounter.create({ ...body, actorId: user.id });
+    const validatedBody = validate(createEncounterSchema, body);
+    const encounterObject = await models.Encounter.create({ ...validatedBody, actorId: user.id });
 
     if (body.dietIds) {
       const dietIds = JSON.parse(body.dietIds);
@@ -204,13 +209,15 @@ encounter.post(
     const encounterObject = await models.Encounter.findByPk(id);
     if (!encounterObject) throw new NotFoundError();
 
-    const { orderingClinicianId, comments, pharmacyOrderPrescriptions } = body;
+    const { orderingClinicianId, comments, isDischargePrescription, pharmacyOrderPrescriptions } =
+      body;
 
     const result = await db.transaction(async () => {
       const pharmacyOrder = await models.PharmacyOrder.create({
         orderingClinicianId,
-        comments,
         encounterId: id,
+        comments,
+        isDischargePrescription,
       });
 
       await models.PharmacyOrderPrescription.bulkCreate(
@@ -266,7 +273,7 @@ encounterRelations.get('/:id/diagnoses', simpleGetList('EncounterDiagnosis', 'en
 encounterRelations.get(
   '/:id/medications',
   asyncHandler(async (req, res) => {
-    const { models, params, query } = req;
+    const { models, params, query, db } = req;
     const { Prescription } = models;
     const { order = 'ASC', orderBy = 'medication.name', rowsPerPage, page, marDate } = query;
 
@@ -374,15 +381,32 @@ encounterRelations.get(
       distinct: true,
     });
 
-    const objects = await Prescription.findAll({
+    const prescriptions = await Prescription.findAll({
       ...baseQueryOptions,
       limit: rowsPerPage,
       offset: page && rowsPerPage ? page * rowsPerPage : undefined,
     });
 
-    const data = objects.map(x => x.forResponse());
+    let responseData = prescriptions.map(p => p.forResponse());
+    if (responseData.length > 0) {
+      const prescriptionIds = responseData.map(p => p.id);
+      const [pharmacyOrderPrescriptions] = await db.query(
+        `
+        SELECT prescription_id, max(created_at) as last_ordered_at 
+        FROM pharmacy_order_prescriptions 
+        WHERE prescription_id IN (:prescriptionIds) and deleted_at is null GROUP BY prescription_id
+      `,
+        { replacements: { prescriptionIds } },
+      );
+      const lastOrderedAts = keyBy(pharmacyOrderPrescriptions, 'prescription_id');
 
-    res.send({ count, data });
+      responseData = responseData.map(p => ({
+        ...p,
+        lastOrderedAt: lastOrderedAts[p.id]?.last_ordered_at?.toISOString(),
+      }));
+    }
+
+    res.send({ count, data: responseData });
   }),
 );
 
@@ -570,6 +594,7 @@ encounterRelations.get(
           survey_responses.deleted_at IS NULL
         AND
           encounters.deleted_at is null
+        AND survey_responses.id NOT IN (SELECT survey_response_id FROM procedure_survey_responses)
         ORDER BY ${sortKey} ${sortDirection}
       `,
       { encounterId, surveyType, surveyIds: permittedSurveyIds },
@@ -595,7 +620,7 @@ async function getAnswersWithHistory(req) {
   const dateDataElement = isVitals
     ? VITALS_DATA_ELEMENT_IDS.dateRecorded
     : CHARTING_DATA_ELEMENT_IDS.dateRecorded;
-  const historyTable = 'vital_logs'; // TODO: Create new model/table and use it here if its not vitals query
+
   // The LIMIT and OFFSET occur in an unusual place in this query
   // So we can't run it through the generic runPaginatedQuery function
   const countResult = await db.query(
@@ -627,6 +652,45 @@ async function getAnswersWithHistory(req) {
   }
 
   const { page = 0, rowsPerPage = 10 } = query;
+  const vitalsHistorySelect = `
+    SELECT
+      vl.answer_id,
+      ARRAY_AGG((
+        JSONB_BUILD_OBJECT(
+          'newValue', vl.new_value,
+          'reasonForChange', vl.reason_for_change,
+          'date', vl.date,
+          'userDisplayName', u.display_name
+        )
+      )) logs
+    FROM survey_response_answers sra
+      INNER JOIN survey_responses sr ON sr.id = sra.response_id
+      LEFT JOIN vital_logs vl ON vl.answer_id = sra.id
+      LEFT JOIN users u ON u.id = vl.recorded_by_id
+    WHERE sr.encounter_id = :encounterId
+      AND sr.deleted_at IS NULL
+    GROUP BY vl.answer_id
+  `;
+  const chartHistorySelect = `
+    SELECT
+      lc.record_id as answer_id,
+      ARRAY_AGG((
+        JSONB_BUILD_OBJECT(
+          'newValue', lc.record_data->>'body',
+          'reasonForChange', lc.reason,
+          'date', TO_CHAR(lc.logged_at, 'YYYY-MM-DD HH24:MI:SS'),
+          'userDisplayName', u.display_name
+        )
+      )) logs
+    FROM survey_response_answers sra
+      INNER JOIN survey_responses sr ON sr.id = sra.response_id
+      LEFT JOIN logs.changes lc ON lc.record_id = sra.id
+      LEFT JOIN users u ON u.id = lc.updated_by_user_id
+    WHERE sr.encounter_id = :encounterId
+      AND sr.deleted_at IS NULL
+      AND lc.table_name = 'survey_response_answers'
+    GROUP BY lc.record_id
+  `;
 
   const result = await db.query(
     `
@@ -645,23 +709,7 @@ async function getAnswersWithHistory(req) {
         ORDER BY body ${order} LIMIT :limit OFFSET :offset
       ),
       history AS (
-        SELECT
-          history_table.answer_id,
-          ARRAY_AGG((
-            JSONB_BUILD_OBJECT(
-              'newValue', history_table.new_value,
-              'reasonForChange', history_table.reason_for_change,
-              'date', history_table.date,
-              'userDisplayName', u.display_name
-            )
-          )) logs
-        FROM survey_response_answers sra
-          INNER JOIN survey_responses sr ON sr.id = sra.response_id
-          LEFT JOIN ${historyTable} history_table ON history_table.answer_id = sra.id
-          LEFT JOIN users u ON u.id = history_table.recorded_by_id
-        WHERE sr.encounter_id = :encounterId
-          AND sr.deleted_at IS NULL
-        GROUP BY history_table.answer_id
+        ${isVitals ? vitalsHistorySelect : chartHistorySelect}
       )
 
       SELECT
@@ -739,7 +787,9 @@ async function getGraphData(req, dateDataElementId) {
     .sort((a, b) => {
       return a.recordedDate > b.recordedDate ? 1 : -1;
     });
-  return data;
+  // Survey ID will be the same for all answers because the
+  // data element ID is unique to the survey
+  return { data, surveyId: dateAnswers[0]?.surveyResponse.surveyId };
 }
 
 encounterRelations.get(
@@ -759,7 +809,7 @@ encounterRelations.get(
   '/:id/graphData/vitals/:dataElementId',
   asyncHandler(async (req, res) => {
     req.checkPermission('list', 'Vitals');
-    const data = await getGraphData(req, VITALS_DATA_ELEMENT_IDS.dateRecorded);
+    const { data } = await getGraphData(req, VITALS_DATA_ELEMENT_IDS.dateRecorded);
 
     res.send({
       count: data.length,
@@ -771,10 +821,9 @@ encounterRelations.get(
 encounterRelations.get(
   '/:id/initialChart$',
   asyncHandler(async (req, res) => {
-    req.checkPermission('list', 'Survey');
     const { models, params } = req;
     const { id: encounterId } = params;
-    const chartSurvey = await models.SurveyResponse.findOne({
+    const chartSurvey = await models.SurveyResponse.findAll({
       attributes: ['survey.*'],
       where: { encounterId },
       include: [
@@ -783,15 +832,21 @@ encounterRelations.get(
           required: true,
           model: models.Survey,
           as: 'survey',
-          where: { surveyType: [SURVEY_TYPES.SIMPLE_CHART, SURVEY_TYPES.COMPLEX_CHART] },
+          where: {
+            surveyType: [SURVEY_TYPES.SIMPLE_CHART, SURVEY_TYPES.COMPLEX_CHART],
+          },
         },
       ],
       order: [['survey', 'name', 'ASC']],
       group: [['survey.id']],
     });
+    req.flagPermissionChecked();
+    const allowedSurvey = chartSurvey.find(response =>
+      req.ability.can('list', subject('Charting', { id: response.survey.id })),
+    );
 
     res.send({
-      data: chartSurvey,
+      data: allowedSurvey,
     });
   }),
 );
@@ -799,8 +854,8 @@ encounterRelations.get(
 encounterRelations.get(
   '/:id/graphData/charts/:dataElementId',
   asyncHandler(async (req, res) => {
-    req.checkPermission('list', 'Charting');
-    const data = await getGraphData(req, CHARTING_DATA_ELEMENT_IDS.dateRecorded);
+    const { data, surveyId } = await getGraphData(req, CHARTING_DATA_ELEMENT_IDS.dateRecorded);
+    req.checkPermission('read', subject('Charting', { id: surveyId }));
 
     res.send({
       count: data.length,
@@ -812,7 +867,8 @@ encounterRelations.get(
 encounterRelations.get(
   '/:id/charts/:surveyId',
   asyncHandler(async (req, res) => {
-    req.checkPermission('list', 'SurveyResponse');
+    const { surveyId } = req.params;
+    req.checkPermission('read', subject('Charting', { id: surveyId }));
     const { count, data } = await getAnswersWithHistory(req);
 
     res.send({
@@ -895,9 +951,8 @@ encounterRelations.get(
   '/:id/charts/:chartSurveyId/chartInstances',
   asyncHandler(async (req, res) => {
     const { db, params } = req;
-    req.checkPermission('list', 'Charting');
-
     const { id: encounterId, chartSurveyId } = params;
+    req.checkPermission('list', subject('Charting', { id: chartSurveyId }));
 
     const results = await db.query(
       `
@@ -952,10 +1007,11 @@ encounterRelations.delete(
   '/:id/chartInstances/:chartInstanceResponseId',
   asyncHandler(async (req, res) => {
     const { db, params, models } = req;
-    req.checkPermission('delete', 'Charting');
-
     const { chartInstanceResponseId } = params;
 
+    const surveyResponse = await models.SurveyResponse.findByPk(chartInstanceResponseId);
+    req.checkPermission('delete', subject('Charting', { id: surveyResponse?.surveyId }));
+
     // all answers will also be soft deleted automatically
     await db.transaction(async () => {
       await models.SurveyResponse.destroy({ where: { id: chartInstanceResponseId } });
